@@ -411,14 +411,18 @@ class IndentedContextualConfig(dict):
 
                     # call the parse() method for this command
 
+                    args_dict = match.groupdict()
+
                     try:
-                        new_config = cmd.parse(
-                            context["config"], **match.groupdict())
+                        new_config = cmd.parse(context["config"], **args_dict)
 
                     except:
                         if explain_exception:
-                            print("%s.parse()" % repr(type(cmd)),
-                                  file=sys.stderr)
+                            print("Exception in %s.parse():"
+                                      % type(cmd).__name__,
+                                  "  *args=" + repr(args_dict),
+                                  "",
+                                  sep="\n", file=sys.stderr)
 
                         raise
 
