@@ -171,6 +171,7 @@ class Cmd_Int_IPOSPFArea(CmdContext_Int):
 
 
 class Cmd_Int_IPOSPFAuth(CmdContext_Int):
+    # TODO: parse to fields
     match = r"ip ospf authentication( (?P<auth>\S+))?"
 
     def parse(self, cfg, auth):
@@ -287,6 +288,56 @@ class Cmd_Int_IPv6MultBdry(CmdContext_Int):
             _SCOPE_BOUNDARIES.get(scope) or int(scope))
 
 
+class Cmd_Int_IPv6NDPfx(CmdContext_Int):
+    match = (
+        r"ipv6 nd prefix (?P<pfx>\S+) "
+        r"("
+            r"(?P<validlife>\d+) (?P<preflife>\d+)"
+            r"|at "
+                r"((?P<validuntil_1d>\d+) (?P<validuntil_1m>[a-z]+)"
+                r"|(?P<validuntil_2m>[a-z]+) (?P<validuntil_2d>\d+))"
+
+                r" (?P<validuntil_y>\d+) (?P<validuntil_time>\d+:\d+)"
+
+                r" "
+                r"((?P<prefuntil_1d>\d+) (?P<prefuntil_1m>[a-z]+)"
+                r"|(?P<prefuntil_2m>[a-z]+) (?P<prefuntil_2d>\d+))"
+
+                r" (?P<prefuntil_y>\d+) (?P<prefuntil_time>\d+:\d+)"
+        r")"
+        r"(?P<no_autocfg> no-autoconfig)?")
+
+    def parse(self, cfg, pfx, validlife, preflife, validuntil_1d,
+              validuntil_1m, validuntil_2m, validuntil_2d, validuntil_y,
+              validuntil_time, prefuntil_1d, prefuntil_1m, prefuntil_2m,
+              prefuntil_2d, prefuntil_y, prefuntil_time, no_autocfg):
+
+        p = {}
+
+        if validlife is not None:
+            p["valid-lifetime"] = int(validlife)
+            p["preferred-lifetime"] = int(preflife)
+
+        else:
+            p["valid-until"] = (
+                "%d %s %d %s"
+                    % (int(validuntil_1d or validuntil_2d),
+                       (validuntil_1m or validuntil_2m).lower(),
+                       int(validuntil_y), validuntil_time))
+
+            p["preferred-until"] = (
+                "%d %s %d %s"
+                    % (int(prefuntil_1d or prefuntil_2d),
+                       (prefuntil_1m or prefuntil_2m).lower(),
+                       int(prefuntil_y), prefuntil_time))
+
+        if no_autocfg:
+            p["no-autoconfig"] = True
+
+        # TODO: should canonicalise prefixes
+        deepsetdefault(cfg, "ipv6-nd-prefix")[pfx.lower()] = p
+
+
 class Cmd_Int_IPv6PIMBSRBdr(CmdContext_Int):
     match = r"ipv6 pim bsr border"
 
