@@ -205,10 +205,12 @@ class CmdContext_VLAN(IndentedContextualCommand):
 
 
 class Cmd_VLAN_Name(CmdContext_VLAN):
-    match = r"name (?P<name>\S+)"
+    # VLAN names can have spaces in but we chop off any trailing spaces
+    # in the parse() method
+    match = r"name (?P<name>.+)"
 
     def parse(self, cfg, name):
-        cfg["name"] = name
+        cfg["name"] = name.strip()
 
 
 
