@@ -11,7 +11,7 @@
 from deepops import deepsetdefault
 import netaddr
 
-from ..utils import interface_canonicalize
+from ..utils import interface_canonicalize, VRF_GLOBAL
 from ...config import IndentedContextualCommand
 
 
@@ -289,13 +289,20 @@ class CmdContext_RtrBGP(IndentedContextualCommand):
     context = "router-bgp"
 
 
+class Cmd_RtrBGP_BGPRtrID(CmdContext_RtrBGP):
+    match = r"bgp router-id (?P<id>\S+)"
+
+    def parse(self, cfg, id):
+        cfg["router-id"] = id
+
+
 class CmdContext_RtrBGP_NbrFallOver(CmdContext_RtrBGP):
     match = (r"neighbor (?P<nbr>\S+) fall-over"
              r" (bfd (?P<bfd>single-hop|multi-hop)|route-map (?P<rtmap>\S+))")
 
     def parse(self, cfg, nbr, bfd, rtmap):
         deepsetdefault(
-            cfg, "neighbor", neighbor_canonicalize(nbr))["fall-over"] = (
+            cfg["neighbor"][neighbor_canonicalize(nbr)])["fall-over"] = (
                 { "bfd": bfd } if bfd else { "route-map": rtmap } )
 
 
@@ -304,33 +311,50 @@ class CmdContext_RtrBGP_NbrPwd(CmdContext_RtrBGP):
 
     def parse(self, cfg, nbr, enc, pwd):
         deepsetdefault(
-            cfg, "neighbor", neighbor_canonicalize(nbr))["password"] = {
+            cfg["neighbor"][neighbor_canonicalize(nbr)])["password"] = {
                 "encryption": int(enc), "password": pwd
             }
 
 
 class CmdContext_RtrBGP_NbrPrGrp(CmdContext_RtrBGP):
+    # this class matches the creation of a peer-group
     match = r"neighbor (?P<nbr>\S+) peer-group"
 
     def parse(self, cfg, nbr):
+        # this creates a neighbor as a peer-group
+        #
+        # unlike most commands that configure neighbors and require them
+        # to exist (by using 'cfg["neighbor"][...(nbr)]), this will
+        # create a new neighbor using a path with deepsetdefault()
         deepsetdefault(
             cfg, "neighbor", neighbor_canonicalize(nbr))["type"] = "peer-group"
 
 
 class CmdContext_RtrBGP_NbrPrGrpMbr(CmdContext_RtrBGP):
+    # this class matches the addition of a member to a peer-group
     match = r"neighbor (?P<nbr>\S+) peer-group (?P<grp>\S+)"
 
     def parse(self, cfg, nbr, grp):
+        # this creates a neighbor as a member of a peer-group
+        #
+        # unlike most commands that configure neighbors and require them
+        # to exist (by using 'cfg["neighbor"][...(nbr)]), this will
+        # create a new neighbor using a path with deepsetdefault()
         deepsetdefault(
             cfg, "neighbor", neighbor_canonicalize(nbr))["peer-group"] = grp
 
 
 class CmdContext_RtrBGP_NbrRemAS(CmdContext_RtrBGP):
-    match = r"neighbor (?P<nbr>\S+) remote-as (?P<rem_as>\d+(\.\d+)?)"
+    match = r"neighbor (?P<nbr>\S+) remote-as (?P<rem_asn>\d+(\.\d+)?)"
 
-    def parse(self, cfg, nbr, rem_as):
+    def parse(self, cfg, nbr, rem_asn):
+        # this creates a new neighbor host
+        #
+        # unlike most commands that configure neighbors and require them
+        # to exist (by using 'cfg["neighbor"][...(nbr)]), this will
+        # create a new neighbor using a path with deepsetdefault()
         deepsetdefault(
-            cfg, "neighbor", neighbor_canonicalize(nbr))["remote-as"] = rem_as
+            cfg, "neighbor", neighbor_canonicalize(nbr))["remote-as"] = rem_asn
 
 
 class CmdContext_RtrBGP_NbrUpdSrc(CmdContext_RtrBGP):
@@ -338,7 +362,7 @@ class CmdContext_RtrBGP_NbrUpdSrc(CmdContext_RtrBGP):
 
     def parse(self, cfg, nbr, int_name):
         deepsetdefault(
-            cfg, "neighbor", neighbor_canonicalize(nbr))["update-source"] = (
+            cfg["neighbor"][neighbor_canonicalize(nbr)])["update-source"] = (
                 int_name)
 
 
@@ -359,9 +383,9 @@ class Cmd_RtrBGP_AF(CmdContext_RtrBGP):
             af += " unicast"
 
         # we put addres families in the global routing table in a VRF
-        # called '_default'
+        # called VRF_GLOBAL
         return deepsetdefault(
-                   cfg, "vrf", vrf or "_default", "address-family", af)
+                   cfg, "vrf", vrf or VRF_GLOBAL, "address-family", af)
 
 
 class CmdContext_RtrBGP_AF(IndentedContextualCommand):
@@ -406,6 +430,30 @@ class CmdContext_RtrBGP_AF_NbrAdvAddPath(CmdContext_RtrBGP_AF):
             a["group-best"] = True
 
 
+
+class Cmd_RtrBGP_AF_NbrAlwAS(CmdContext_RtrBGP_AF):
+    match = r"neighbor (?P<nbr>\S+) allowas-in( (?P<max>\d+))?"
+
+    def parse(self, cfg, nbr, max):
+        # we can't just use None for an empty 'allowas-in' maximum as
+        # this cannot be changed to, as a different type
+        n = deepsetdefault(cfg, "neighbor", neighbor_canonicalize(nbr))
+        a = {}
+        if max is not None:
+            a["max"] = int(max)
+        n["allowas-in"] = a
+
+
+class CmdContext_RtrBGP_AF_NbrFallOver(CmdContext_RtrBGP_AF):
+    match = (r"neighbor (?P<nbr>\S+) fall-over"
+             r" (bfd (?P<bfd>single-hop|multi-hop)|route-map (?P<rtmap>\S+))")
+
+    def parse(self, cfg, nbr, bfd, rtmap):
+        deepsetdefault(
+            cfg["neighbor"][neighbor_canonicalize(nbr)])["fall-over"] = (
+                { "bfd": bfd } if bfd else { "route-map": rtmap } )
+
+
 class CmdContext_RtrBGP_AF_NbrFltLst(CmdContext_RtrBGP_AF):
     match = (r"neighbor (?P<nbr>\S+) filter-list (?P<list_>\d+)"
              r" (?P<dir_>in|out)")
@@ -429,10 +477,14 @@ class CmdContext_RtrBGP_AF_NbrMaxPfx(CmdContext_RtrBGP_AF):
 
 
 class CmdContext_RtrBGP_AF_NbrNHSelf(CmdContext_RtrBGP_AF):
-    match = r"neighbor (?P<nbr>\S+) next-hop-self"
+    match = r"neighbor (?P<nbr>\S+) next-hop-self(?P<all> all)?"
 
-    def parse(self, cfg, nbr):
-        deepsetdefault(cfg, "neighbor", nbr)["next-hop-self"] = True
+    def parse(self, cfg, nbr, all):
+        n = deepsetdefault(cfg, "neighbor", neighbor_canonicalize(nbr))
+        h = {}
+        if all:
+            h["all"] = True
+        n["next-hop-self"] = h
 
 
 class CmdContext_RtrBGP_AF_NbrPrGrp(CmdContext_RtrBGP_AF):
@@ -459,6 +511,35 @@ class CmdContext_RtrBGP_AF_NbrPfxLst(CmdContext_RtrBGP_AF):
         deepsetdefault(cfg, "neighbor", nbr, "prefix-list")[dir_] = list_
 
 
+class CmdContext_RtrBGP_AF_NbrPwd(CmdContext_RtrBGP_AF):
+    match = r"neighbor (?P<nbr>\S+) password( (?P<enc>\d)) (?P<pwd>\S+)"
+
+    def parse(self, cfg, nbr, enc, pwd):
+        deepsetdefault(
+            cfg["neighbor"][neighbor_canonicalize(nbr)])["password"] = {
+                "encryption": int(enc), "password": pwd
+            }
+
+
+class CmdContext_RtrBGP_AF_NbrRemAs(CmdContext_RtrBGP_AF):
+    match = r"neighbor (?P<nbr>\S+) remote-as (?P<rem_asn>\d+(\.\d+)?)"
+
+    def parse(self, cfg, nbr, rem_asn):
+        deepsetdefault(
+            cfg, "neighbor", neighbor_canonicalize(nbr))["remote-as"] = rem_asn
+
+
+class CmdContext_RtrBGP_AF_NbrRemPrivAS(CmdContext_RtrBGP_AF):
+    match = r"neighbor (?P<nbr>\S+) remove-private-as(?P<all> all)?"
+
+    def parse(self, cfg, nbr, all):
+        n = deepsetdefault(cfg, "neighbor", neighbor_canonicalize(nbr))
+        r = {}
+        if all:
+            r["all"] = True
+        n["remove-private-as"] = r
+
+
 class CmdContext_RtrBGP_AF_NbrRtMap(CmdContext_RtrBGP_AF):
     match = r"neighbor (?P<nbr>\S+) route-map (?P<rtmap>\S+) (?P<dir_>in|out)"
 
@@ -495,7 +576,21 @@ class CmdContext_RtrBGP_AF_NbrSoftRecfg(CmdContext_RtrBGP_AF):
                 "soft-reconfiguration"] = "inbound"
 
 
-class CmdContext_RtrBGP_AF_Redist_Simple(CmdContext_RtrBGP_AF):
+class CmdContext_RtrBGP_AF_MaxPaths(CmdContext_RtrBGP_AF):
+    match = r"maximum-paths (?P<paths>\d+)"
+
+    def parse(self, cfg, paths):
+        cfg["maximum-paths"] = int(paths)
+
+
+class CmdContext_RtrBGP_AF_MaxPathsIBGP(CmdContext_RtrBGP_AF):
+    match = r"maximum-paths ibgp (?P<paths>\d+)"
+
+    def parse(self, cfg, paths):
+        cfg["maximum-paths-ibgp"] = int(paths)
+
+
+class CmdContext_RtrBGP_AF_Redist_Plain(CmdContext_RtrBGP_AF):
     match = (r"redistribute (?P<proto>static|connected)"
              r"( route-map (?P<rtmap>\S+))?( metric (?P<met>\d+))?")
 
@@ -507,7 +602,7 @@ class CmdContext_RtrBGP_AF_Redist_Simple(CmdContext_RtrBGP_AF):
             r["metric"] = int(met)
 
 
-class CmdContext_RtrBGP_AF_Redist_ID(CmdContext_RtrBGP_AF):
+class CmdContext_RtrBGP_AF_Redist_OSPF(CmdContext_RtrBGP_AF):
     match = (r"redistribute (?P<proto>ospf|ospfv3) (?P<proc>\d+)"
              r"( route-map (?P<rtmap>\S+))?( metric (?P<met>\d+))?")
 
