@@ -36,7 +36,10 @@ class Cvt_Int(DiffConvert):
         return "no interface " + int_name
 
     def add(self, new, int_name):
-        return "interface " + int_name
+        # physical interfaces don't need creating (it doesn't hurt but
+        # it's not necessary and makes the unit tests more cumbersome)
+        if not is_int_physical(int_name):
+            return "interface " + int_name
 
 
 class DiffConvert_Int(DiffConvert):
@@ -99,10 +102,11 @@ class Cvt_Int_CDPEna(DiffConvert_Int):
     cmd = "cdp-enable",
 
     def remove(self, old, int_name):
-        # if the 'cdp enable' option is not present, that doesn't mean
-        # it's disabled but just that it's not specified, so we assume
-        # the default is for it to be enabled
-        return self.enter(int_name) + [" cdp enable"]
+        # if the 'cdp enable' option is not present, that just means
+        # it's reverted to the default setting of enabled - if it wasn't
+        # previously enabled, we do that
+        if not old:
+            return self.enter(int_name) + [" cdp enable"]
 
     def update(self, old, upd, new, int_name):
         return self.enter(int_name) + [
@@ -145,13 +149,13 @@ class Cvt_Int_IPAccGrp(DiffConvert_Int):
     cmd = "ip-access-group", None
 
     def remove(self, old, int_name, dir_):
-        return self.enter(int_name), [" no ip access-group " + dir_]
+        return self.enter(int_name) + [" no ip access-group " + dir_]
 
     def update(self, old, upd, new, int_name, dir_):
         return self.enter(int_name) + [" ip access-group %s %s" % (new, dir_)]
 
 
-# ip-address ...
+# ip address ...
 
 
 class Cvt_Int_IPAddr(DiffConvert_Int):
@@ -176,6 +180,27 @@ class Cvt_Int_IPAddrSec(DiffConvert_Int):
         return self.enter(int_name) + [" ip address %s secondary" % addr]
 
 
+# ...
+
+
+class Cvt_Int_IPFlowMon(DiffConvert_Int):
+    cmd = "ip-flow-monitor", None
+
+    def remove(self, old, int_name, dir_):
+        return self.enter(int_name) + [
+                   " no ip flow monitor %s %s" % (old, dir_)]
+
+    def update(self, old, upd, new, int_name, dir_):
+        l = self.enter(int_name)
+
+        # we must remove the old flow monitor before setting a new one
+        if old:
+            l += [" no ip flow monitor %s %s" % (old, dir_)]
+
+        l += [" ip flow monitor %s %s" % (new, dir_)]
+        return l
+
+
 # ip helper-address ...
 #
 # This command is odd in that changing VRF causes it to be modified but
@@ -235,24 +260,6 @@ class Cvt_VRFPost_Int_IPHlprAddr(DiffConvert_Int_IPHlprAddr):
 # ...
 
 
-class Cvt_Int_IPFlowMon(DiffConvert_Int):
-    cmd = "ip-flow-monitor", None
-
-    def remove(self, old, int_name, dir_):
-        return self.enter(int_name) + [
-                   " no ip flow monitor %s %s" % (old, dir_)]
-
-    def update(self, old, upd, new, int_name, dir_):
-        l = self.enter(int_name)
-
-        # we must remove the old flow monitor before setting a new one
-        if old:
-            l += [" no ip flow monitor %s %s" % (old, dir_)]
-
-        l += [" ip flow monitor %s %s" % (new, dir_)]
-        return l
-
-
 class Cvt_Int_IPIGMPVer(DiffConvert_Int):
     cmd = "ip-igmp-version",
 
@@ -390,9 +397,15 @@ class Cvt_Int_IPPolicyRtMap(DiffConvert_Int):
 class Cvt_Int_IPProxyARP(DiffConvert_Int):
     cmd = "ip-proxy-arp",
 
+    def remove(self, old, int_name):
+        # if proxy ARP was disabled and we remove it, we're reverting to
+        # the default of enabled
+        if not old:
+            return self.enter(int_name) + [" ip proxy-arp"]
+
     def update(self, old, upd, new, int_name):
         return self.enter(int_name) + [
-                   " " + ("" if upd else "no ") + "ip proxy-arp"]
+                   " " + ("" if new else "no ") + "ip proxy-arp"]
 
 
 class Cvt_Int_IPVerifyUni(DiffConvert_Int):
@@ -426,7 +439,24 @@ class Cvt_Int_IPv6MultBdry(DiffConvert_Int):
         return self.enter(int_name) + [" no ipv6 multicast boundary scope"]
 
     def update(self, old, upd, new, int_name):
-        return self.enter(int_name) + [" ipv6 multicast boundary scope " + new]
+        return self.enter(int_name) + [
+            " ipv6 multicast boundary scope %d" % new]
+
+
+class Cvt_Int_IPv6NDPfx(DiffConvert_Int):
+    cmd = "ipv6-nd-prefix", None
+
+    def remove(self, old, int_name, pfx):
+        return self.enter(int_name) + [" no ipv6 nd prefix " + pfx]
+
+    def update(self, old, upd, new, int_name, pfx):
+        if "valid-lifetime" in new:
+            # we're using relative number of seconds
+            e = "%d %d" % (new["valid-lifetime"], new["preferred-lifetime"])
+        else:
+            # we're using absolute 'until' date/time
+            e = "at %s %s" % (new["valid-until"], new["preferred-until"])
+        return self.enter(int_name) + [" ipv6 nd prefix %s %s" % (pfx, e)]
 
 
 class Cvt_Int_IPv6PIMBSRBdr(DiffConvert_Int):
@@ -471,30 +501,6 @@ class Cvt_Int_IPv6VerifyUni(DiffConvert_Int):
         return self.enter(int_name) + [" ipv6 verify unicast " + new]
 
 
-class Cvt_Int_ServPol(DiffConvert_Int):
-    cmd = "service-policy", None, None
-
-    def _cmd(self, type_, dir_, name):
-        return ("service-policy"
-                + ((" type " + type_) if type_ else "")
-                + " " + dir_ + " " + name)
-
-    def remove(self, old, int_name, dir_, type_):
-        return self.enter(int_name) + [" no " + self._cmd(type_, dir_, old)]
-
-    def update(self, old, upd, new, int_name, dir_, type_):
-        l = self.enter(int_name)
-
-        # we cannot just replace a service-policy: we need to remove the
-        # old one first
-        if old:
-            l.append(" no " + self._cmd(type_, dir_, old))
-
-        l.append(" " + self._cmd(type_, dir_, new))
-
-        return l
-
-
 # mpls ...
 
 
@@ -502,10 +508,10 @@ class Cvt_Int_MPLSIP(DiffConvert_Int):
     cmd = "mpls-ip",
 
     def remove(self, old, int_name):
-        return self.enter(int_name) + [" no mpls ip "]
+        return self.enter(int_name) + [" no mpls ip"]
 
     def update(self, old, upd, new, int_name):
-        return self.enter(int_name) + [" mpls ip "]
+        return self.enter(int_name) + [" mpls ip"]
 
 
 # mtu ...
@@ -524,7 +530,8 @@ class Cvt_Int_MTU(DiffConvert_Int):
         return self.enter(int_name) + [" no mtu"]
 
     def update(self, old, upd, new, int_name):
-        if old and "channel-group" in old:
+        # TODO: handle what happens if interface converting to channel-group
+        if new and "channel-group" in new:
             return None
 
         return self.enter(int_name) + [" mtu " + str(self.get_ext(new))]
@@ -587,6 +594,33 @@ class Cvt_Int_OSPFv3Net(DiffConvert_Int):
         return self.enter(int_name) + [" ospfv3 network " + new]
 
 
+# ...
+
+
+class Cvt_Int_ServPol(DiffConvert_Int):
+    cmd = "service-policy", None, None
+
+    def _cmd(self, type_, dir_, name):
+        return ("service-policy"
+                + ((" type " + type_) if type_ else "")
+                + " " + dir_ + " " + name)
+
+    def remove(self, old, int_name, dir_, type_):
+        return self.enter(int_name) + [" no " + self._cmd(type_, dir_, old)]
+
+    def update(self, old, upd, new, int_name, dir_, type_):
+        l = self.enter(int_name)
+
+        # we cannot just replace a service-policy: we need to remove the
+        # old one first
+        if old:
+            l.append(" no " + self._cmd(type_, dir_, old))
+
+        l.append(" " + self._cmd(type_, dir_, new))
+
+        return l
+
+
 # standby ...
 
 
@@ -674,6 +708,9 @@ class Cvt_Int_StandbyVer(DiffConvert_Int):
     def update(self, old, upd, new, int_name):
         # only set this here if we're switching to version >= 2 (so we
         # can potentially use any high-numbered groups)
+        #
+        # version 1 is the default so is effectively removing 'standby
+        # version' in the Cvt_Int_NoStandbyVer class
         if new < 2:
             return
         return self.enter(int_name) + [" standby version " + str(new)]
