@@ -114,14 +114,14 @@ class Cvt_TrackList(DiffConvert_TrackCreate):
         if new["type"] == "boolean":
             return ["track %d list boolean %s" % (track_num, new["op"])]
 
-        return ValueError("invalid track list type:" + l["type"])
+        return ValueError("unhandled track list type:" + l["type"])
 
 
 class Cvt_TrackRoute(DiffConvert_TrackCreate):
     cmd = "route",
 
     def update(self, old, upd, new, track_num):
-        return ["track %d %s %s %s"
+        return ["track %d %s route %s %s"
                     % (track_num, new["proto"], new["net"], new["measure"])]
 
 
@@ -136,15 +136,26 @@ class DiffConvert_TrackSub(DiffConvert):
 class Cvt_Track_Delay(DiffConvert_TrackSub):
     cmd = "delay",
 
-    def remove(self, old, obj):
-        return self.enter(obj) + [" no delay"]
+    def truncate(self, old, rem, new, obj):
+        # delays cannot individually be removed from a tracking object;
+        # only all delays at the same time
+        #
+        # to remove a delay, we have to clear them all and re-add the
+        # one we're keeping
+        #
+        # we only re-add it here, though, if new value is the same as
+        # the old, otherwise we handle the change separately in update()
+        c = self.enter(obj) + [" no delay"]
+        for dir_ in sorted(new or []):
+            if (dir_ in (old or [])) and (old[dir_] == new[dir_]):
+                c.append(" delay %s %d" % (dir_, new[dir_]))
+        return c
 
     def update(self, old, upd, new, obj):
-        d = ""
-        for dir_ in new:
-            if dir_ in new:
-                d += " %s %d" % (dir_, new[dir_])
-        return self.enter(obj) + [" delay" + d]
+        c = self.enter(obj)
+        for dir_ in sorted(upd):
+            c.append(" delay %s %d" % (dir_, new[dir_]))
+        return c
 
 
 class Cvt_Track_IPVRF(DiffConvert_TrackSub):
