@@ -25,6 +25,20 @@ from ..config import IndentedContextualCommand, IndentedContextualConfig
 
 
 
+# --- constants ---
+
+
+
+# there are times when we store global routing table (outside a VRF, or
+# in the 'global' or 'default' VRF) under a VRF name for convenience
+#
+# this is the name we use for that and is defined here once, to be used
+# consistently
+
+VRF_GLOBAL = "_global"
+
+
+
 # --- interface functions ---
 
 
